#!/usr/local/bin/perl
#
# Copyright (c) 2005 Erik Mugele.  All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
# THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# Designed and written by Erik Mugele, 2004-2005
# http://www.teuton.org/~erik
# Version 1.0
#
# CGI to change a Subversion password.
#

use CGI;

$eol="\n";
$dirdelim="/";
$repodir="/usr/local/svn".$dirdelim."repos";
$pathsuffix="conf".$dirdelim."passwd";


sub printHTMLhead {
  print $query->header;
  print <<EOF;
  <html>
  <head>
  <title>Subversion Password Management System</title>
  </head>
  <body>
  <center><h2><font face='arial'>Subversion Password Management System</font></h2></center>
EOF
}

sub printHTMLfoot {
  print <<EOF;
  </body>
  </html>
EOF
}

sub printform {
  opendir(dh,$repodir) or die "Directory $repodir does not exist.\n";
  foreach $i (sort readdir(dh)) {
    push(@repolist,$i);
  }
  closedir(dh);

  print <<EOF;
  <font face='arial'>Notes:</font>
  <ul>
  <li><font face='arial'>Passwords should not contain whitespace.</font></li>
  <li><font face='arial'>Passwords are stored on the server in plain text and are readable by system administrators.</font></li>
  </ul>
  <HR>
  <form method='post' action='$thisscript'>
  <table>
  <tr>
  <td align='right'><font face='arial'>Repository: </font></td>
  <td>
  <select name='repo'>
EOF
  foreach $i (@repolist) {
    if ($i !~ m/^\./) {
      print "<option value='$i'>$i</option>\n";
    }
  }
  print <<EOF;
  </select>
  </td>
  </tr>
  <tr>
  <td align='right'><font face='arial'>Username: </font></td>
  <td><input type='text' name='login'></td>
  </tr>
  <tr>
  <td align='right'><font face='arial'>Current Password: </font</td>
  <td><input type='password' name='opasswd'></td>
  </tr>
  <tr>
  <td align='right'><font face='arial'>New Password: </font</td>
  <td><input type='password' name='npasswd'></td>
  </tr>
  <tr>
  <td align='right'><font face='arial'>Verify Password: </font</td>
  <td><input type='password' name='vpasswd'></td>
  </tr>
  </table>
  <input type='submit' name='submit' value='Change Password'>
EOF
}

$thisscript=(split($dirdelim,$0))[-1];
$query = new CGI;
if ($query->param("submit")) {
  if (($query->param("repo")) &&
      ($query->param("login")) && 
      ($query->param("opasswd")) &&
      ($query->param("npasswd")) &&
      ($query->param("vpasswd"))) {
    $errormessage="";
    $repo=$query->param("repo");
    $user=$query->param("login");
    $opass=$query->param("opasswd");
    $npass=$query->param("npasswd");
    $passwd_file = $repodir.$dirdelim.$repo.$dirdelim.$pathsuffix;
    $passwd_backup = "$passwd_file~";
    if ($query->param("npasswd") ne $query->param("vpasswd")) {
      $errormessage .= "New password and verified password don't match.";
    } else {
      if (not stat $passwd_file) {
        $errormessage .= "Invalid repository and/or password file.  Someone needs to check in to this.";
      }
    }
    if ($errormessage eq "") { 
      open (fd, $passwd_file) or die "Can't open $passwd_file.\n";
      while (<fd>) {
        push(@passwd_list,$_);
        next if (/^#/ || /^$/ || /^\[/ || /^\s$/);
        ($tmp_user,$tmp_pass)=split(/=/,$_);
        $tmp_user =~ s/\s//g;
        $tmp_pass =~ s/\s//g;
        $passwd_hash{$tmp_user}=$tmp_pass;
      }
      close(fd) or die "Can't close $passwd_file~.\n";

      if ((not exists $passwd_hash{$user}) || ($passwd_hash{$user} ne $opass)) {
        $errormessage = "Invalid username and/or password.";
      } else {
        # Back up the passwd file.
        open (fd, ">$passwd_backup") or die "Can't open $passwd_backup.\n";
        foreach $i (@passwd_list) {
          print fd $i;
        }
        close (fd) or die "Can't close $passwd_backup.\n";
        open (fd, ">$passwd_file") or die "Can't open $passwd_file for writing.\n";
        foreach $i (@passwd_list) {
          # Remove the last \r if we are running native on a Windows system.
          if (($i =~ m/^$user/) && ($i =~ m/$passwd_hash{$user}$/)) {
            print fd "$user = $npass\n";
          } else {
            print fd $i;
          }
        }
        close (fd) or die "Can't close $passwd_file.\n";
      }
    }
  } else {
    $errormessage = "Invalid parameters or empty fields.";
  }
  if ($errormessage ne "") {
    printHTMLhead;
    print "<font face='arial' color='red'>$errormessage</font><br><br>$eol";
    printform;
    printHTMLfoot;
  } else {
    printHTMLhead;
    print "<font face='arial' color='green'>Password for <b>$user</b> has been successfully changed in <b>$repo</b> repository.</font><br><br>$eol";
    printform;
    printHTMLfoot;
  }
} else {
  printHTMLhead;
  printform;
  printHTMLfoot;
}
 
